#ifndef AMDir_INCLUDED
#define AMDir_INCLUDED

/******************************************************************
*
* Software Systems Research Laboratory
* Department of Electrical Engineering and Computer Science
* University of Michigan at Ann Arbor
* 1301 Beal Avenue, Ann Arbor, MI 48105
* 
*
* Copyright (c) 1999 The Regents of The University of Michigan.
* All rights reserved.
* 
* Contact: pdmcdan@eecs.umich.edu
*******************************************************************/

//////////////////////////////////////////////////////////////////////////////////
//
//
//   File : AMDir.h
//
//   Created : 27-Dec-00
//   By : Patrick McDaniel
//
//   Description : This is the base file for directory entries in AMirD.
//
//   *** Change History ***
//
//   27-Dec-00 Created File : DPM
//
//////////////////////////////////////////////////////////////////////////////////

// *** Include Files ***

// Project Include Files
#include <Queue.h>
#include <AMEntry.h>

// *** Definitions *** 

// Class Definition AMDir
class AMDir : public AMEntry {

  public: // Public Interface

    //
    // Constructors and Destructors

    // Default Constructor
    AMDir( AMFsys *fs, AMEntry *par, const char * file )
      : AMEntry( fs, par, file )
    {
    }

    // Destructor
    virtual ~AMDir( void )
    {
    }

    //
    // Access Methods

    // Return a reference to the queue of entries
    Queue & Entries( void )
    {
        return( subEntries );
    }

    // 
    // Member Methods

    // what type of entry is this? (ISFILE/ISDIR)
    virtual EntryType entryType( void )
    {
        return( ISDIR );
    }

    virtual Boolean scanEntryInfo( Boolean recursive );
      // Scan, from local filesystem, information about entry

    virtual Boolean remove( void );
      // Remove an entry from the filesystem

    // Process the begining of an xfer
    virtual Boolean xferBegin( void )
    {
        // Bad - should never happen
        ATK_ASSERT( 0, "xfer begin on directory." );
        return( FALSE );
    }

    // Process a completed transfer
    virtual Boolean xferComplete( Boolean suc )
    {
        // Bad - should never happen
        ATK_ASSERT( 0, "xfer complete on directory." );
        return( FALSE );
    }

    virtual Boolean doAnnounce( Buffer & info, BufferList *annce );
      // Advertise announcement information.

    virtual Boolean reconcileAnnounce( Buffer & info );
      // Reconcile an announcement with received information.

    virtual Boolean processAnnouncement( Buffer & info );
      // Process an update announcement

    //
    // Static Methods

    static Boolean makePath( AString & base, AString & rel, 
                             unsigned long mode, AuditLog *log = NULL );
      // create the directories towards the path

  private: // Private Interface and Data

    // 
    // Class Attributes

    Queue subEntries;
      // This is a queue of sub-entries.

};
// End Class Definition AMDir

#endif
