#ifndef AMFile_INCLUDED
#define AMFile_INCLUDED

/******************************************************************
*
* Software Systems Research Laboratory
* Department of Electrical Engineering and Computer Science
* University of Michigan at Ann Arbor
* 1301 Beal Avenue, Ann Arbor, MI 48105
* 
*
* Copyright (c) 1999 The Regents of The University of Michigan.
* All rights reserved.
* 
* Contact: pdmcdan@eecs.umich.edu
*******************************************************************/

//////////////////////////////////////////////////////////////////////////////////
//
//
//   File : AMFile.h
//
//   Created : 27-Dec-00
//   By : Patrick McDaniel
//
//   Description : This is a file entry type used for
//                 importing/exporting files within AMirD.
//
//   *** Change History ***
//
//   27-Dec-00 Created File : DPM
//
//////////////////////////////////////////////////////////////////////////////////

// *** Include Files ***
#include <AtkTimer.h>

// Project Include Files
#include <AMEntry.h>

// *** Definitions *** 

// Class Definition AMFile
class AMFile : public AMEntry {

  public: // Public Interface

    //
    // Constructors and Destructors

    // Default Constructor
    AMFile( AMFsys *fs, AMEntry *par, const char * file )
      : AMEntry( fs, par, file ),
        exRequest( TRUE ),
        upReqTimer( NULL ),
        lastHashed( 0 )
    {
    }

    // Destructor
    virtual ~AMFile( void )
    {
        if ( upReqTimer != NULL ) delete upReqTimer;
    }

    //
    // Access Methods

    // Return the file size
    unsigned long fileSize( void )
    {
        return( size );
    }

    // Return the explicit request flag
    Boolean & explicitRequest( void )
    {
        return( exRequest );
    }

    // Return a pointer to the update request
    AtkTimer * & UpReqTimer( void )
    {
        return( upReqTimer );
    }

    //
    // Abstract Methods

    // what type of entry is this? (ISFILE/ISDIR)
    virtual EntryType entryType( void )
    {
        return( ISFILE );
    }

    virtual Boolean scanEntryInfo( Boolean recursive );
      // Scan, from local filesystem, information about entry

    virtual Boolean remove( void );
      // Remove an entry from the filesystem

    virtual Boolean xferBegin( void );
      // Process the begining of an xfer

    virtual Boolean xferComplete( Boolean suc );
      // Process a completed transfer

    // 
    // Member Methods

    virtual Boolean doAnnounce( Buffer & info, BufferList *annce );
      // Advertise announcement information.

    virtual Boolean reconcileAnnounce( Buffer & info );
      // Reconcile an announcement with received information.

    void GetFileHash (Buffer *buf)
    {
        *buf = fileHash;
    }

    Boolean HashFile( void );
      // Hash the file

  private: // Private Interface and Data

    //
    // Private Interfaces

    // 
    // Class Attributes

    unsigned long size;
      // This is the size (in bytes) of the file.

    Boolean exRequest;
      // Should this file be explicitly requests.

    AtkTimer * upReqTimer;
      // This is the update request timer for this file.

    time_t lastHashed;
      // This is the time the file was hashed last.

    Buffer fileHash;
      // This is a hash of the file contents.

};
// End Class Definition AMFile

#endif
