#ifndef AMImpFsys_INCLUDED
#define AMImpFsys_INCLUDED

/******************************************************************
*
* Software Systems Research Laboratory
* Department of Electrical Engineering and Computer Science
* University of Michigan at Ann Arbor
* 1301 Beal Avenue, Ann Arbor, MI 48105
* 
*
* Copyright (c) 1999 The Regents of The University of Michigan.
* All rights reserved.
* 
* Contact: pdmcdan@eecs.umich.edu
*******************************************************************/

//////////////////////////////////////////////////////////////////////////////////
//
//
//   File : AMImpFsys.h
//
//   Created : 27-Dec-00
//   By : Patrick McDaniel
//
//   Description : This is the object used to record the imported
//                 filesystem data.
//
//   *** Change History ***
//
//   27-Dec-00 Created File : DPM
//
//////////////////////////////////////////////////////////////////////////////////

// *** Include Files ***

// Project Include Files
#include <AMFsys.h>

// *** Definitions *** 
#define UPDATE_REQUEST_PERIOD 30000 // Request Every 30 seconds

// Class Definition AMImpFsys
class AMImpFsys : public AMFsys {

  public: // Public Interface

    //
    // Constructors and Destructors

    // Attribute Constructor
    AMImpFsys( AString & ex, AString & lpath, AString & epath, 
               AString & lpol, AString & ath, AuditLog *log = NULL )
      : AMFsys( ex, lpath, lpol, ath, log ),
        exportedPath( epath )
    {
    }

    // Destructor
    virtual ~AMImpFsys( void )
    {
    }

    //
    // Access Methods

    // This is the exported path
    AString & ExportPath( void )
    {
        return( exportedPath );
    }

    // 
    // Member Methods

    virtual Boolean processUpdateRequest( Buffer * buf );
      // Process an incoming update request

    virtual Buffer * createFileId( AMFile *fl );
      // Create a file id buffer

    virtual AMFile * findFileById( Buffer *buf );
      // Find the file associated with this ID

    // Is this an exported filesystem
    virtual Boolean Exported( void )
    {
        return( FALSE );
    }

    Boolean processAnnouncement( Buffer & buf );
      // Process an incoming announcment

    void getUpdateRequests(BufferList *reqs, Random *random);
      // create a queue of update requests

  private: // Private Interface and Data

    // 
    // Class Attributes

    AString exportedPath;
      // This is the path exported by the exporter.

};

typedef AMImpFsys* AMImpFsysPtr;

class AMImpFsysList : public list<AMImpFsysPtr>
{
  public:

    // find the given item and set the iterator. 
    // The item matches if the AString value is the same.
    // Return NULL and set iterator to end() if not found.
    AMImpFsys *find (AMImpFsys *fs, iterator *iter)
    {
        AString search = *fs;
        for (*iter = begin(); *iter != end(); (*iter)++)
        {
            AMImpFsys *f = *(*iter);
            if (search == *f)
            {
                return f;
            }
        }
        return NULL;
    }


    // find the given item.  Return NULL on not found.
    AMImpFsys *find (AMImpFsys *fs)
    {
        iterator iter;
        return find (fs, &iter);
    }


    // find the given item by file name.  Return NULL on not found.
    AMImpFsys *find (AString *fileName)
    {
        iterator iter;
        for (iter = begin(); iter != end(); (iter)++)
        {
            AMImpFsys *f = *iter;
            if (*fileName == *f)
            {
                return f;
            }
        }
        return NULL;
    }

    // find the given item and erase it if found.  Do
    // not do a delete on the object.  Return it if found,
    // else return NULL.
    AMImpFsys *deleteItem (AMImpFsys *fs)
    {
        iterator iter;
        AMImpFsys *ret;     // return value

        ret = find (fs, &iter);
        if (ret != NULL)
        {
            erase (iter);
        }
        return ret;
    }
};

// End Class Definition AMImpFsys

#endif
