#ifndef AMXferMem_INCLUDED
#define AMXferMem_INCLUDED

/******************************************************************
*
* Software Systems Research Laboratory
* Department of Electrical Engineering and Computer Science
* University of Michigan at Ann Arbor
* 1301 Beal Avenue, Ann Arbor, MI 48105
* 
*
* Copyright (c) 1999 The Regents of The University of Michigan.
* All rights reserved.
* 
* Contact: pdmcdan@eecs.umich.edu
*******************************************************************/

//////////////////////////////////////////////////////////////////////////////////
//
//
//   File : AMXferMem.h
//
//   Created : 13-Oct-00
//   By : Patrick McDaniel
//
//   Description : This class encapsulates a member of the broadcast
//                 file xfer protocol.
//
//   *** Change History ***
//
//   13-Oct-00 Created File : DPM
//
//////////////////////////////////////////////////////////////////////////////////

// *** Include Files ***

// Project Include Files
#include <BitVector.h>
#include <AEntity.h>

// Class Definition AMXferMem
class AMXferMem : public BitVector {

  public: // Public Interface

    //
    // Constructors and Destructors

    // Attribute Constructor
    AMXferMem( AEntity *ent, unsigned long len )
      : BitVector (len),
        entity( ent )
    {
    }

    // Destructor
    virtual ~AMXferMem( void )
    {
    }

    //
    // Access Methods

    // Return the entity
    AEntity * Entity( void )
    {
        return( entity );
    }

    // 
    // Member Methods

    // Return true of all packets for this block have been acked
    Boolean allAcked( void )
    {
        return( onBits() == Length() );
    }

    // Acknowledge a set of packets
    BitVector & acknowledge( BitVector & vec )
    {
        return( OR(vec) );
    }

    // Reset the packet to all unacknowledge
    void reset( void )
    {
        SetVector( FALSE );
    }

    //
    // Listable Functions

    // Get a hash value of the object
    virtual unsigned short hashValue( void )
    {
        return( entity->hashValue() );
    }

    // Compare this object to one like it
    // Returns : < 0 if local less than object
    //           = 0 if local equal to object
    //           > 0 if local greater than object
    // NOTE: Just compares the pointers
    virtual int compare( Listable *obj )
    {
       // Use entity compare
       return( entity->compare(obj) );
    }

    // This object compares a listable value/identifier to the local
    // object.
    // Returns : < 0 if local less than object
    //           = 0 if local equal to object
    //           > 0 if local greater than object
    virtual int compareValue( void *val )
    {
       // Use entity compareValue
       return( entity->compareValue(val) );
    }

    // Return a string describing the object
    virtual const char * toString( void )
    {
        static AString str, str2;
        str.sprintf( "%s [%s]", (char *)*entity, (char *)vectorString(str2) );
        return( (char *)str );
    }

  private: // Private Interface and Data

    // 
    // Class Attributes

    AEntity *entity;
      // This is the entity object associated with the member.

};
// End Class Definition AMXferMem

#endif

