#ifndef AMirDPolicyImpl_INCLUDED
#define AMirDPolicyImpl_INCLUDED

/******************************************************************
*
* Software Systems Research Laboratory
* Department of Electrical Engineering and Computer Science
* University of Michigan at Ann Arbor
* 1301 Beal Avenue, Ann Arbor, MI 48105
* 
*
* Copyright (c) 1999 The Regents of The University of Michigan.
* All rights reserved.
* 
* Contact: pdmcdan@eecs.umich.edu
*******************************************************************/

//////////////////////////////////////////////////////////////////////////////////
//
//
//   File : AMirDPolicyImpl.h
//
//   Created : 22-May-01
//   By : Patrick McDaniel
//
//   Description : This class is the policy implementor used by the
//                 control and download groups of AMirD.  It currently
//                 implements the folliwing policy conditionals.
//
//   *** Change History ***
//
//   22-May-01 Created File : DPM
//
//////////////////////////////////////////////////////////////////////////////////

// *** Include Files ***

// Project Include Files
#include <APolicyImplementor.h>

// *** Forward References ***
class AMirD;
class AMFxfer;

// *** Definitions *** 
#define AM_MAX_SENS_DIRS 2
#define AM_MAX_SENS_SITES 1

// Class Definition AMirDPolicyImpl
class AMirDPolicyImpl : public APolicyImplementor {

  public: // Public Type Definitions
 
    // Identify the group types
    typedef enum AMirDGroupTypeEnum {

        AMIRD_CONTROL_GROUP  = 0,  // Control Group
        AMIRD_DOWNLOAD_GROUP = 1   // Download Group

    } AMirDGroupType;

  public: // Public Interface

    //
    // Constructors and Destructors

    // Control Group Constructor
    AMirDPolicyImpl( AMirD * amd, AuditLog *lg = NULL )
      : groupType( AMIRD_CONTROL_GROUP ),
        cntrlGroup( amd ),
        dwnldGroup( NULL ),
        log( lg )
    {
    }

    // Download Group Constructor
    AMirDPolicyImpl( AMFxfer * fx, AuditLog *lg = NULL )
      : groupType( AMIRD_DOWNLOAD_GROUP ),
        cntrlGroup( NULL ),
        dwnldGroup( fx ),
        log( lg )
    {
    }

    // Destructor
    virtual ~AMirDPolicyImpl( void )
    {
    }

    // 
    // Member Methods

    virtual Boolean EvaluateCondition( ACondEval & cond );
      // Evaluate a conditional

    //
    // Debug Functions

    void testPredicates( void );
      // test some of the implentor predicates

  private: // Private Interface and Data

    //
    // Private Interfaces

    // Test whether this is a sensitive file/directory
    Boolean isSensitive( AString & fl )
    {
        // If starts with any of the sensitive directories
        for ( int i=0; i<AM_MAX_SENS_DIRS; i++ )
            if ( fl.StartsWith(AMIRD_SENSITIVE_DIRECTORIES[i]) == TRUE )
                return( TRUE );

        // Nope, not sensitive
        return( FALSE );
    }

    // 
    // Class Attributes

    AMirDGroupType groupType;
      // Flag identifying whether this is a control or download group.

    AMirD * cntrlGroup;
      // This is the control group object.

    AMFxfer * dwnldGroup;
      // This is the download group object.

    AString groupClass;
      // This is the type of group implemented

    AuditLog * log;
      // This is the log to which to write debug information

    //
    // Static Class Attributes

    static const char * AMIRD_SENSITIVE_DIRECTORIES[AM_MAX_SENS_DIRS];
      // This is the list of sensitive directories

    static const char * AMIRD_SENSITIVE_SITES[AM_MAX_SENS_SITES];
      // This is the list of sensitive directories

};
// End Class Definition AMirDPolicyImpl

#endif
